@@ -1,6 +1,6 @@
 ;;; brltty.el --- Interface to BRLTTY
 
-;; Copyright (C) 2004, 2005, 2006, 2007 Brailcom, o.p.s.
+;; Copyright (C) 2004, 2005, 2006, 2007, 2008 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -100,13 +100,14 @@ available, from the  environment variable CONTROLVT."
     (7 . "Invalid size")
     (8 . "Connection refused")
     (9 . "Operation not supported")
-    (10 . " Getaddrinfo error")
-    (11 . " Libc error")
-    (12 . " Couldn't find out the tty number")
-    (13 . " Bad protocol version")
-    (14 . " Unexpected end of file")
-    (15 . " Key file empty")
-    (16 . " Packet returned by driver too large")))
+    (10 . "Getaddrinfo error")
+    (11 . "Libc error")
+    (12 . "Couldn't find out the tty number")
+    (13 . "Bad protocol version")
+    (14 . "Unexpected end of file")
+    (15 . "Key file empty")
+    (16 . "Packet returned by driver too large")
+    (17 . "Authentication failed")))
 
 (defconst brltty--packet-types
   '(;; commands
@@ -117,6 +118,8 @@ available, from the  environment variable CONTROLVT."
     (leavetty . ?L)
     (getdisplaysize . ?s)
     (write . ?w)
+    (ignorekeyranges . ?m)
+    (acceptkeyranges . ?u)
     ;; answers
     (ack . ?A)
     (err . ?e)
@@ -352,6 +355,9 @@ available, from the  environment variable CONTROLVT."
                         :key #'(lambda (data)
                                  (cond 
 				  ((integerp data) 4)
+                                  ((consp data)
+                                   (ecase (car data)
+                                     (integer64 8)))
 				  (t (length data))))))
         (process (brltty--connection-process connection)))
     (when process
@@ -378,6 +384,16 @@ available, from the  environment variable CONTROLVT."
                     (process-send-string
                      process
                      (string-make-unibyte (format "%c" (aref data i))))))
+                 ((consp data)
+                  (destructuring-bind (n1 n2 n3) (cdr data)
+                    (ecase (car data)
+                      (integer64
+                       (process-send-string
+                        process
+                        (string-make-unibyte (format "%c%c%c%c%c%c%c%c"
+                                                     (/ n1 256) (% n1 256)
+                                                     (/ n2 (* 256 256)) (% (/ n2 256) 256) (% n2 256)
+                                                     (/ n3 (* 256 256)) (% (/ n3 256) 256) (% n3 256))))))))
                  (t
                   (process-send-string process data))))
               (when answer
@@ -423,22 +439,21 @@ respectively."
         (if (> version 7)
             (progn
               (unless (member version brltty--supported-protocol-versions)
-                (setq version (apply max brltty--supported-protocol-versions)))
-              (setq auth-methods (brltty--send-packet connection 'auth
-                                                      'version version))
-              (cond
-               ((memq 'none auth-methods)
-                (brltty--send-packet
-                 connection 'ack 'auth
-                 (car (rassoc 'none brltty--authentication-codes))))
-               ((memq 'key auth-methods)
-                (brltty--send-packet
-                 connection 'ack 'auth
-                 (car (rassoc 'key brltty--authentication-codes))
-                 (brltty--authentication-key)))
-               (t
-                (signal 'brltty-connection-error
-                        "No supported BrlAPI authentication method"))))
+                (setq version (apply 'max brltty--supported-protocol-versions)))
+              (let ((auth-methods (brltty--send-packet connection 'auth 'version version)))
+                (cond
+                 ((memq 'none auth-methods)
+                  (brltty--send-packet
+                   connection 'ack 'auth
+                   (car (rassoc 'none brltty--authentication-codes))))
+                 ((memq 'key auth-methods)
+                  (brltty--send-packet
+                   connection 'ack 'auth
+                   (car (rassoc 'key brltty--authentication-codes))
+                   (brltty--authentication-key)))
+                 (t
+                  (signal 'brltty-connection-error
+                          "No supported BrlAPI authentication method")))))
           ;; No server initial message, assume protocol version 7 (older
           ;; versions are not supported)
           (let ((answer (brltty--send-packet
@@ -447,6 +462,7 @@ respectively."
             (when (equal answer brltty--protocol-version-error)
               (signal 'brltty-error answer))))
         (setf (brltty--connection-protocol-version connection) version)
+        (brltty--update-terminal-spec connection)
         connection)
     (error
      (signal 'brltty-connection-error err))))
@@ -481,7 +497,6 @@ TEXT is encoded in the coding given by `brltty-coding' before it is sent.
 CURSOR, if non-nil, is a position of the cursor on the display, starting
 from 0."
   (when connection
-    (brltty--update-terminal-spec connection)
     (let ((display-width (car (brltty-display-size connection))))
       (when display-width
         (let* ((text* (if (> (length text) display-width)
@@ -511,6 +526,22 @@ from 0."
               (setf arguments (append arguments (list (vector (length charset)) (upcase charset))))))
           (apply #'brltty--send-packet arguments))))))
 
+(defun brltty-ignore-keys (connection)
+  "Let BrlTTY handle all keys itself."
+  (when connection
+    (brltty--send-packet connection nil 'ignorekeyranges '(integer64 0 0 0) '(integer64 #xFFFF #xFFFFFF #xFFFFFF))))
+
+(defun brltty-accept-keys (connection &optional keys)
+  "Let BrlTTY send all keys to us.
+If optional argument KEYS is non-nil, allow to send us only the given keys.
+Then KEYS must be a list of key codes represented by integer triplets."
+  (when connection
+    (let ((key-ranges (if keys
+                          (mapcar #'(lambda (key) (cons (cons 'integer64 key) (cons 'integer64 key))) keys)
+                        (list (cons '(integer64 0 0 0) '(integer64 #xFFFF #xFFFFFF #xFFFFFF))))))
+      (dolist (range key-ranges)
+        (brltty--send-packet connection nil 'acceptkeyranges (car range) (cdr range))))))
+
 
 ;;; Announce
 
