@@ -1,6 +1,6 @@
 ;;; speechd-speak.el --- simple speechd-el based Emacs client
 
-;; Copyright (C) 2003, 2004, 2005, 2006, 2007 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -269,7 +269,7 @@ created."
                         (const :tag "Non-buffers" nil)
                         (const :tag "Minibuffer" :value :minibuffer)
                         (symbol :tag "Major mode" :value fundamental-mode)
-                        (string :tag "Buffer name" :value "")
+                        (regexp :tag "Buffer name regexp" :value "")
                         (restricted-sexp :tag "Function call"
                                          :match-alternatives (listp)))
                 :value-type (string :tag "Connection name"))
@@ -436,7 +436,10 @@ This variable is reset to nil before each command in pre-command-hook.")
                                 (setq specs (cdr specs))))
                             result)
                           ;; buffer name
-                          (assoc (buffer-name) speechd-speak-connections)
+                          (let ((buffer-name (buffer-name)))
+                            (assoc-if (lambda (key)
+                                        (when (stringp key) (string-match key buffer-name)))
+                                      speechd-speak-connections))
                           ;; minibuffer
                           (and (speechd-speak--in-minibuffer-p)
                                (assoc :minibuffer speechd-speak-connections))
@@ -572,8 +575,9 @@ If the prefix argument is given, output the line only from the current point
 to the end of the line."
   (interactive "P")
   (speechd-speak--interactive
-   (let ((beg (if rest-only (point) (line-beginning-position)))
-         (end (line-end-position)))
+   (let* ((inhibit-field-text-motion t)
+          (beg (if rest-only (point) (line-beginning-position)))
+          (end (line-end-position)))
      (when speechd-speak-separator-regexp
        (save-match-data
          (save-excursion
@@ -1700,6 +1704,13 @@ Only single characters are allowed in the keymap.")
 (speechd-speak--command-feedback indent-for-tab-command after
   (speechd-speak--speak-current-column))
 
+(defvar speechd-speak--handle-braille-keys nil)
+(defun speechd-speak-toggle-braille-keys ()
+  (interactive)
+  (speechd-out-set 'brltty-accept-keys (not speechd-speak--handle-braille-keys))
+  (setq speechd-speak--handle-braille-keys (not speechd-speak--handle-braille-keys))
+  (message "Braille keys handled by %s" (if speechd-speak--handle-braille-keys "speechd-el" "BrlTTY")))
+
 
 ;;; Spelling
 
@@ -1970,6 +1981,7 @@ The following symbols are valid state change identifiers: `buffer-name',
 (define-key speechd-speak-mode-map "\C-r" 'speechd-speak-read-rectangle)
 (define-key speechd-speak-mode-map "\C-s" 'speechd-speak)
 (define-key speechd-speak-mode-map "\C-x" 'speechd-unspeak)
+(define-key speechd-speak-mode-map "\C-bk" 'speechd-speak-toggle-braille-keys)
 (dotimes (i 9)
   (define-key speechd-speak-mode-map (format "%s" (1+ i))
               'speechd-speak-key-set-predefined-rate))
