@@ -1,6 +1,6 @@
 ;;; speechd.el --- Library for accessing Speech Dispatcher
 
-;; Copyright (C) 2003, 2004, 2005, 2006, 2007 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -67,7 +67,7 @@
 (defcustom speechd-spdsend nil
   "If string, it names the spdsend binary to be used to talk to SSIP.
 If the new version of accept-process-output is available or the value of this
-variable is nil, Emacs talks to SSIP directly."
+variable is nil, Emacs talks to an SSIP server (Speech Dispatcher) directly."
   :type '(choice (const :tag "Don't use spdsend" nil)
                  (string :tag "spdsend binary"))
   :group 'speechd)
@@ -117,6 +117,10 @@ is a symbol under which the voice will be accessed and PARAMETERS is an alist
 of parameter identifiers and parameter values.  Valid parameter names are the
 following symbols: language, gender, age, style, name, rate, pitch, volume,
 punctuation-mode, capital-character-mode, message-priority, output-module.
+Please note that any parameter entry present will change the corresponding
+parameter, even if the parameter value is nil or empty; if you don't want to
+change the parameter in any way by the voice, don't put it to the list (and
+don't enable its entry in customize).
 
 Name is a string identifying Speech Dispatcher voice name.  If it is not given,
 the parameters gender, age, and style are considered to select a Speech
@@ -305,6 +309,7 @@ current voice."
   host
   port
   (failure-p nil)
+  (last-try-time nil)
   process
   (process-output "")
   (paused-p nil)
@@ -319,6 +324,8 @@ current voice."
 (defvar speechd--connections (make-hash-table :test 'equal)
   "Hash table mapping client names to `speechd-connection' instances.")
 
+(defvar speechd--retry-time 1.0)
+
 
 ;;; Utilities
 
@@ -338,6 +345,11 @@ current voice."
 
 (defmacro speechd--with-current-connection (&rest body)
   `(let ((connection (speechd--connection)))
+     (when (and (speechd--connection-failure-p connection)
+                (>= (- (float-time) (speechd--connection-last-try-time connection))
+                    speechd--retry-time))
+       (ignore-errors (speechd-reopen t))
+       (setq connection (speechd--connection)))
      (unless (speechd--connection-failure-p connection)
        ,@body)))
 
@@ -593,8 +605,9 @@ Return the opened connection on success, nil otherwise."
   "Close all speechd connections."
   (speechd--iterate-clients (speechd-close)))
 
-(defun speechd-reopen ()
-  "Close and open again all the connections to speechd."
+(defun speechd-reopen (&optional quiet)
+  "Close and open again all the connections to speechd.
+If QUIET is non-nil, don't echo success report."
   (interactive)
   (let ((number-of-connections 0)
 	(number-of-failures 0))
@@ -604,8 +617,9 @@ Return the opened connection on success, nil otherwise."
 		       (incf number-of-connections)
 		     (incf number-of-failures))))
 	     speechd--connections)
-    (message (format "%d connections successfully reopened, %d failures"
-		     number-of-connections number-of-failures))))
+    (unless quiet
+      (message (format "%d connections successfully reopened, %d failures"
+                       number-of-connections number-of-failures)))))
 
 
 ;;; Process communication functions
@@ -621,6 +635,7 @@ Return the opened connection on success, nil otherwise."
 (defun speechd--permanent-connection-failure (connection)
   (speechd--close-connection connection)
   (setf (speechd--connection-failure-p connection) t
+        (speechd--connection-last-try-time connection) (float-time)
 	(speechd--connection-paused-p connection) nil
 	(speechd--connection-parameters connection) ()))
 
